/************************************************************************
* Modulo : manta_in                   Archivo: manta_in.h         *
* Fecha creacion : 17/Jan/92                                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
*   Declaracion de constantes, tipos y macros                           *
*   del proyecto.                                                       *
*   Declaracion de los prototipos de la funciones selectoras del modulo *
*                                                                       *
************************************************************************/
 
       /*------------------------------------------------------+
       |                      CONSTANTES                       |
       +------------------------------------------------------*/
 
#define MAX_OP         20  /*-- Numero maximo de opciones del menu --*/
#define LONG_CAD       80  /*-- Longitud maxima de una cadena      --*/
#define LONG_OP        50  /*-- Longitud maxima de una opcion      --*/
#define NO_OPCION      -1  /*-- No se digito ninguna opcion        --*/

 
       /*------------------------------------------------------+
       |                       MENSAJES                        |
       +------------------------------------------------------*/

#include "manta.msg"


       /*------------------------------------------------------+
       |                         TIPOS                         |
       +------------------------------------------------------*/

typedef struct opcion {
      char    nombre[LONG_OP]; /*-- Cadena del nombre de una opcion --*/
      int     longitud;        /*-- Longitud de la opcion           --*/
}  *opcion;

typedef struct menu{
      int     num_op;  /*-- Numero de opciones del menu              --*/
      int     act_op;  /*-- Opcion actual del menu                   --*/
      char    titulo[2*LONG_OP]; /*-- Titulo del menu               --*/
      opcion  opciones[MAX_OP]; /*-- Opciones del menu               --*/ 
} *menu;

typedef struct objetos {
      char    nom_tipo[LONG_OP]; /*-- Nombre del tipo                --*/
      char    nom_var[LONG_OP];  /*-- Nombre de la variable          --*/
      union   {
          ptr     obj;           /*-- Contiene un objeto complejo    --*/
          int     NAT;           /*-- Contiene un objeto de tipo NAT --*/
          int     BOOL;          /*-- Contiene un objeto de tipo BOOL--*/
      } u;
      struct objetos *sig; /*-- Siguiente objeto de la lista         --*/
}*objetos;

typedef struct funciones {
    char tipo[LONG_OP];    /*-- Nombre del tipo --*/
    ptr  f_impr;           /*-- Funcion de impresion del tipo --*/  
    ptr  f_salv;           /*-- Funcion de salvar del tipo --*/
    ptr  f_carg;           /*-- Funcion de cargar del tipo --*/
    ptr  f_copi;           /*-- Funcion de copiar del tipo --*/
    ptr  f_borr;           /*-- Funcion de borrar del tipo --*/
    struct funciones *sig; /*-- Siguiente tipo --*/
}*funciones;

typedef void   (*tipo_fimpr)(ptr); /*-- Funcion de impresion --*/

typedef void   (*tipo_fsalv)(FILE *,ptr); /*-- Funcion de salvar    --*/

typedef ptr    (*tipo_fcarg)(FILE *); /*-- Funcion de cargar    --*/

typedef ptr    (*tipo_fcopi)(ptr); /*-- Funcion de copiar    --*/

typedef void   (*tipo_fborr)(ptr*); /*-- Funcion de borrar    --*/
 
        /*------------------------------------------------------+
        |   Variables externas                                  |
        +------------------------------------------------------*/

extern
objetos objs;      /*-- Lista de objetos del ambiente --*/

extern
funciones func;    /*-- Lista de funciones de los tipos del ambiente --*/

extern
menu    ambiente;  /*-- Menu con opciones para salvar y cargar el ambiente */

extern
menu    interfaces; /*-- Menu con opciones para salvar, cargar, copiar y
                         borrar objetos del ambiente  */

extern
int     tad_error; /*-- Guarda un codigo de error, en caso de falla--*/
 

        /*------------------------------------------------------+
        |   Prototipos de las funciones externas del proyecto   |
        +------------------------------------------------------*/
  
extern 
opcion  crear_opcion(char nombre[]);

extern 
menu  	crear_menu();

extern 
void    adic_opcion(menu m, opcion o);

extern
void    pintar_menu(menu m);

extern
short   es_prefijo(char comando[],char cad_opcion[]);

extern
int     admon_menu(menu m);

extern
int     opcion_valida(menu m, char comando[]);

extern
void    imprimir_obj_prim(ptr o,char nombre[]);


extern
void    leer_guardar_obj(ptr o, char tit[],char tipo[]);

extern
objetos nuevo_obj(ptr o, char tipo[], char nombre[]);

extern
void    guardar_obj(ptr o, char tipo[], char nombre[]);

extern
void    borrar_obj(objetos *o);

extern
objetos leer_obj_prim(char tit[], char tipo[]);

extern
void    salvar_escribir(char tit[],char tipo[]);  

extern
void    escriba_cadena(char mensaje[], int n);

extern
void    leer_cadena(char mensaje[],char cadena[]);

extern
objetos leer_buscar_obj(char tit[], char tipo[]);

extern  
void    listar_objs(void);

extern
void    borrar_lista_objs(void); 

extern
ptr     buscar_funcion(char tipo[], int clase);

extern
void	adicionar_funcion(char tipo[], ptr f_impr, ptr f_salv,
			  ptr f_carg, ptr f_copi, ptr f_borr); 

extern 
void	leer_impr_obj(char tipo[]);

extern
void	leer_salv_obj(char tipo[]); 

extern
void	salvar_objs(void);

extern
void	leer_carg_obj(char tipo[]);

extern
void	cargar_objs(void);

extern
void	leer_copi_obj(char tipo[]);

extern
void	leer_borr_obj(char tipo[]);

extern
ptr     ERROR(char mensaje[]);

extern
void    crear_menu_ambiente();

extern
void	admon_menu_ambiente(void);

extern
void    crear_menu_interfaces();

extern
void	admon_menu_interfaces(char tipo[]);



